function [p1,u1,p2,u2,inlier1,inlier2,p01,u01,p02,u02] = CDMS(pc,uc,d_th,dp_th,Nr)
fprintf('------- CDMS -------\n')
Skew = @(x) [0 -x(3) x(2);x(3) 0 -x(1);-x(2) x(1) 0];
n = length(pc);

%% Multiple-Solution RANSAC

Pi = zeros(3,2*Nr);
ui = zeros(3,2*Nr);
Ni = zeros(1,2*Nr);
k = 0;
for i = 1:Nr
    idx = randperm(n,4);
    [p1,u1,p2,u2] = fourLineIntersect(pc(:,idx),uc(:,idx));
    p1 = real(p1);
    p2 = real(p2);
    u1 = real(u1);
    u2 = real(u2);
    N1 = NoI(p1,u1,pc,uc,d_th);
    N2 = NoI(p2,u2,pc,uc,d_th);
    if sum(isnan(p1)) > 0 || sum(isnan(p2)) > 0 ...
       || sum(isnan(u1)) > 0 || sum(isnan(u2)) > 0
        continue
    end
    k = k + 1;
    Pi(:,2*i-1) = p1;
    ui(:,2*i-1) = u1;
    Ni(2*i-1) = N1;
    Pi(:,2*i-0) = p2;
    ui(:,2*i-0) = u2;
    Ni(2*i-0) = N2;
end
Nr = k;

[~,idx] = max(Ni);
p01 = Pi(:,idx);
u01 = ui(:,idx);

Di = zeros(1,Nr);
for i = 1:Nr
    Di(i) = dlineline(Pi(:,i),ui(:,i),p01,u01);
end
[centers,U] = fcm(Di',2,[NaN NaN NaN 0]);
if centers(1) < centers(2)
    idx_g2 = U(1,:) < U(2,:);
else
    idx_g2 = U(1,:) > U(2,:);
end

[~,idx2] = max(Ni(idx_g2));
idx2 = find(cumsum(idx_g2)==idx2,1);
p02 = Pi(:,idx2);
u02 = ui(:,idx2);

%% Coordinate Descent Algorithm for Finding the First Line 
p1 = p01;
u1 = u01;

Nitr = 1000;
for itr = 1:Nitr
    pp = PerpendicularPoints(pc,uc,p1,u1);
    A = Skew(u1)*pp-repmat(Skew(u1)*p1,1,length(uc));
    Di = sqrt(sum(A.^2,1));
    if itr < 10
        inlier1 = Di < dp_th;
    end
    [p1,u1] = BestLine(pp(:,inlier1));
end


%% Coordinate Descent Algorithm for Finding the Second Line 
p2 = p02;
u2 = u02;
X0 = [p2;u2]';

Di = zeros(1,length(uc));
for i = 1:length(uc)
    Di(i) = dlineline(pc(:,i),uc(:,i),p2,u2);
end
inlier2 = Di < dp_th;

p0 = X0(1:3)';
n0 = X0(4:6)';

p2 = p0;
u2 = n0;
Nitr = 1000;
for itr = 1:Nitr
    pp = PerpendicularPoints(pc,uc,p2,u2);
    A = Skew(u2)*pp-repmat(Skew(u2)*p2,1,length(uc));
    Di = sqrt(sum(A.^2,1));
    if itr < 10
        inlier2 = Di < dp_th;
    end
    [p2,u2] = BestLine(pp(:,inlier2));
end

% f = dlines([p2' u2'],pc,uc);
% f = sum(f.^2);
% fprintf('Final cost of the second solution: %f\n',f);
% fprintf('Number of inliers: %d\n',sum(inlier2));
