function [p1,t1,p2,t2] = fourLineIntersect(P,u)
    ua = u(:,1);
    ub = u(:,2);
    uc = u(:,3);
    ud = u(:,4);
    pa = P(:,1);
    pb = P(:,2);
    pc = P(:,3);
    pd = P(:,4);

    Xb = pb - ub;
    Yb = pb;
    Zb = pb + ub;

    Xb2 = Xb - ua'*(Xb-pa)*ua;
    Yb2 = Yb - ua'*(Yb-pa)*ua;
    Zb2 = Zb - ua'*(Zb-pa)*ua;

    pc2 = pc - ua'*(pc-pa)*ua;
    uc2 = uc - ua'*uc*ua;

    Xc2 = LinesIntersectionPoint(Xb2,Xb2-pa,pc2,uc2);
    Yc2 = LinesIntersectionPoint(Yb2,Yb2-pa,pc2,uc2);
    Zc2 = LinesIntersectionPoint(Zb2,Zb2-pa,pc2,uc2);

    Xc = LinesIntersectionPoint(Xc2,ua,pc,uc);
    Yc = LinesIntersectionPoint(Yc2,ua,pc,uc);
    Zc = LinesIntersectionPoint(Zc2,ua,pc,uc);

    Xb3 = Xb - ud'*(Xb-pd)*ud;
    Yb3 = Yb - ud'*(Yb-pd)*ud;
    Zb3 = Zb - ud'*(Zb-pd)*ud;

    Xc3 = Xc - ud'*(Xc-pd)*ud;
    Yc3 = Yc - ud'*(Yc-pd)*ud;
    Zc3 = Zc - ud'*(Zc-pd)*ud;

    d = cross(ud,randn(3,1));
    d = d/norm(d);
    O = pd + d;
    Bx = pd + 2*(Xb3-pd)'*d/norm(Xb3-pd)^2*(Xb3-pd);
    By = pd + 2*(Yb3-pd)'*d/norm(Yb3-pd)^2*(Yb3-pd);
    Bz = pd + 2*(Zb3-pd)'*d/norm(Zb3-pd)^2*(Zb3-pd);
    Cx = pd + 2*(Xc3-pd)'*d/norm(Xc3-pd)^2*(Xc3-pd);
    Cy = pd + 2*(Yc3-pd)'*d/norm(Yc3-pd)^2*(Yc3-pd);
    Cz = pd + 2*(Zc3-pd)'*d/norm(Zc3-pd)^2*(Zc3-pd);

    Y = LinesIntersectionPoint(Bx,Cy-Bx,Cx,By-Cx);
    Z = LinesIntersectionPoint(Bx,Cz-Bx,Cx,Bz-Cx);

    pp = Y + (O-Y)'*(Z-Y)/norm(Z-Y)^2*(Z-Y);
    T = pp + sqrt(1-norm(pp-O)^2)/norm(Z-Y)*(Z-Y);
    U = pp - sqrt(1-norm(pp-O)^2)/norm(Z-Y)*(Z-Y);

    Tb3 = LinesIntersectionPoint(T,pd-T,Xb3,Yb3-Xb3);
    Tc3 = LinesIntersectionPoint(T,pd-T,Xc3,Yc3-Xc3);
    Ub3 = LinesIntersectionPoint(U,pd-U,Xb3,Yb3-Xb3);
    Uc3 = LinesIntersectionPoint(U,pd-U,Xc3,Yc3-Xc3);

    Tb = LinesIntersectionPoint(Tb3,ud,pb,ub);
    Tc = LinesIntersectionPoint(Tc3,ud,pc,uc);
    Ub = LinesIntersectionPoint(Ub3,ud,pb,ub);
    Uc = LinesIntersectionPoint(Uc3,ud,pc,uc);
    
    p1 = Tb;
    t1 = (Tc-Tb)/norm(Tc-Tb);
    p2 = Ub;
    t2 = (Uc-Ub)/norm(Uc-Ub);
end